@@ -178,10 +178,10 @@ $(document).ready(function() {
 										vmailtest.handleRemoteJsep({ jsep: jsep });
 									}
 								},
-								onlocalstream: function(stream) {
+								onlocaltrack: function(track, on) {
 									// We're not going to attach the local audio stream
 								},
-								onremotestream: function(stream) {
+								onremotetrack: function(track, mid, on) {
 									// We're not going to receive anything from the plugin
 								},
 								oncleanup: function() {
